/*
  * FreeModbus Libary: LPC214X Port
  * Copyright (C) 2007 Tiago Prado Lone <tiago@maxwellbohr.com.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: portserial.c,v 1.1 2007/04/24 23:15:18 wolti Exp $
 */

#include "M451Series.h"
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"


     static void     prvvUARTTxReadyISR( void );
     static void     prvvUARTRxISR( void );

/* ----------------------- Start implementation -----------------------------*/
void  vMBPortSerialEnable( BOOL xRxEnable, BOOL xTxEnable )
{
	__disable_irq();
	
	if( xRxEnable )
    {
       UART_ENABLE_INT(UART0, UART_INTEN_RDAIEN_Msk); 
    }
    else
    {
  		  UART_DISABLE_INT(UART0, UART_INTEN_RDAIEN_Msk);
    }
    if( xTxEnable )
    {
         UART_ENABLE_INT(UART0, UART_INTEN_THREIEN_Msk);
    }
    else
    {
      UART_DISABLE_INT(UART0, UART_INTEN_THREIEN_Msk);
    }
	
	
	NVIC_EnableIRQ(UART0_IRQn);
	__enable_irq();	 
}

void
vMBPortClose( void )
{
		/* Disable Interrupt */
      UART_DISABLE_INT(UART0, UART_INTEN_RDAIEN_Msk | UART_INTEN_THREIEN_Msk);

	/* Disable I & F bit */ 
			NVIC_DisableIRQ(UART0_IRQn);
}

BOOL
xMBPortSerialInit( UCHAR ucPORT, ULONG ulBaudRate, UCHAR ucDataBits, eMBParity eParity )
{
	if (ucPORT!=0) return FALSE;
    UART_Open(UART0, ulBaudRate);
    return TRUE;
		 
	
}

BOOL
xMBPortSerialPutByte( CHAR ucByte )
{
    UART_WRITE(UART0 , ucByte);				
    return TRUE;
}

BOOL
xMBPortSerialGetByte( CHAR * pucByte )
{
    *pucByte=UART_READ(UART0 );	    
    return TRUE;
}


void UART0_IRQHandler(void)
{
		uint32_t u32IntSts = UART0->INTSTS;
  if(u32IntSts & UART_INTSTS_THREINT_Msk){
	          prvvUARTTxReadyISR();//жӳ 
	}
	else if(u32IntSts & UART_INTSTS_RDAINT_Msk){
				while(UART_IS_RX_READY(UART0))
	        	prvvUARTRxISR();//жӳ 
	}
	
}



/* 
 * Create an interrupt handler for the transmit buffer empty interrupt
 * (or an equivalent) for your target processor. This function should then
 * call pxMBFrameCBTransmitterEmpty( ) which tells the protocol stack that
 * a new character can be sent. The protocol stack will then call 
 * xMBPortSerialPutByte( ) to send the character.
 */
static void
prvvUARTTxReadyISR( void )
{
    pxMBFrameCBTransmitterEmpty(  );
}

/* 
 * Create an interrupt handler for the receive interrupt for your target
 * processor. This function should then call pxMBFrameCBByteReceived( ). The
 * protocol stack will then call xMBPortSerialGetByte( ) to retrieve the
 * character.
 */
static void
prvvUARTRxISR( void )
{
    pxMBFrameCBByteReceived(  );
}
